@@ -928,6 +928,27 @@ class ImagesController(object):
         except exception.NotAuthenticated as e:
             raise webob.exc.HTTPUnauthorized(explanation=e.msg)
 
+    def _validate_hashing_data(self, val_data):
+        if 'os_hash_value' in val_data:
+            try:
+                hashval = bytearray.fromhex(val_data['os_hash_value'])
+            except ValueError:
+                msg = (_("os_hash_value (%s) is not a valid hexadecimal"
+                         " value") % (val_data['os_hash_value']))
+                raise webob.exc.HTTPBadRequest(explanation=msg)
+
+            hash_algo = val_data.get('os_hash_algo',
+                                     CONF['hashing_algorithm'])
+            want_size = hashlib.new(hash_algo).digest_size
+            if len(hashval) != want_size:
+                msg = (_("os_hash_value: (%(value)s) is not the correct "
+                         "size for (%(algo)s) "
+                         "(should be (%(want)d) bytes)") %
+                       {'value': val_data['os_hash_value'],
+                        'algo': hash_algo,
+                        'want': want_size})
+                raise webob.exc.HTTPBadRequest(explanation=msg)
+
     def _validate_validation_data(self, image, locations):
         val_data = {}
         for loc in locations:
@@ -1108,6 +1129,117 @@ class ImagesController(object):
             raise webob.exc.HTTPInternalServerError(
                 explanation=encodeutils.exception_to_unicode(e))
 
+    def add_location(self, req, image_id, body):
+        url = body.get('url')
+        validation_data = body.get('validation_data', {})
+        image_repo = self.gateway.get_repo(req.context)
+        ctxt = req.context
+        stole_lock_from_task = None
+        task_factory = self.gateway.get_task_factory(ctxt)
+        task_repo = self.gateway.get_task_repo(ctxt)
+        try:
+            image = image_repo.get(image_id)
+            if image.status != 'queued':
+                msg = _("It's not allowed to add locations if image status is "
+                        "%s.") % image.status
+                raise webob.exc.HTTPConflict(explanation=msg)
+
+            api_pol = api_policy.ImageAPIPolicy(req.context, image,
+                                                self.policy)
+            api_pol.add_location()
+
+            roles = list(set(req.context.roles + req.context.service_roles))
+            if 'service' not in roles:
+                # NOTE(pdeore): Add location API is disabled for other stores
+                # than http
+                if not utils.is_http_store_configured(url):
+                    msg = _("http store must be enabled to use location API"
+                            " by normal user.")
+                    raise exception.Forbidden(msg)
+
+            if validation_data is not None:
+                self._validate_hashing_data(validation_data)
+
+            if 'os_glance_import_task' in image.extra_properties:
+                # NOTE(pdeore): This will raise exception.Conflict if the
+                # lock is present and valid, or return if absent or invalid.
+                stole_lock_from_task = self._enforce_import_lock(req, image)
+
+            task_input = {'image_id': image_id,
+                          'loc_url': url,
+                          'validation_data': validation_data}
+
+            executor_factory = self.gateway.get_task_executor_factory(
+                ctxt)
+            add_location_task = task_factory.new_task(
+                task_type='location_import',
+                owner=ctxt.owner,
+                task_input=task_input,
+                image_id=image_id,
+                user_id=ctxt.user_id,
+                request_id=ctxt.request_id)
+
+            try:
+                # NOTE(pdeore): Try to grab the lock for this task
+                image_repo.set_property_atomic(image, 'os_glance_import_task',
+                                               add_location_task.task_id)
+            except exception.Duplicate:
+                msg = (_("New operation on image '%s' is not "
+                         "permitted as prior operation is still "
+                         "in progress") % image_id)
+                raise exception.Conflict(msg)
+
+            # NOTE(pdeore): We now have the import lock on this image.
+            # If we busted the lock above and have a reference to that
+            # task, try to clean up the import status information left
+            # over from that execution.
+            if stole_lock_from_task:
+                self._cleanup_stale_task_progress(image_repo, image,
+                                                  stole_lock_from_task)
+
+            task_repo.add(add_location_task)
+            task_executor = executor_factory.new_task_executor(ctxt)
+            pool = common.get_thread_pool("tasks_pool")
+            pool.spawn(add_location_task.run, task_executor)
+        except exception.Conflict as e:
+            raise webob.exc.HTTPConflict(explanation=e.msg)
+        except exception.NotFound as e:
+            raise webob.exc.HTTPNotFound(explanation=e.msg)
+        except exception.Forbidden as e:
+            LOG.debug("User not permitted to add location to image '%s'",
+                      image_id)
+            raise webob.exc.HTTPForbidden(explanation=e.msg)
+        except exception.NotAuthenticated as e:
+            raise webob.exc.HTTPUnauthorized(explanation=e.msg)
+        except ValueError as e:
+            raise webob.exc.HTTPBadRequest(
+                explanation=encodeutils.exception_to_unicode(e))
+
+        return image_id
+
+    def get_locations(self, req, image_id):
+        image_repo = self.gateway.get_repo(req.context)
+        try:
+            image = image_repo.get(image_id)
+
+            # NOTE(pdeore): This is the right place to check whether user
+            # have permission to get the image locations
+            api_pol = api_policy.ImageAPIPolicy(req.context, image,
+                                                self.policy)
+            api_pol.get_locations()
+            locations = list(image.locations)
+
+            for loc in locations:
+                loc.pop('id', None)
+                loc.pop('status', None)
+        except exception.NotFound as e:
+            raise webob.exc.HTTPNotFound(explanation=e.msg)
+        except exception.Forbidden as e:
+            LOG.debug("User not permitted to get the image locations.")
+            raise webob.exc.HTTPForbidden(explanation=e.msg)
+
+        return locations
+
 
 class RequestDeserializer(wsgi.JSONRequestDeserializer):
 
@@ -1134,9 +1266,10 @@ class RequestDeserializer(wsgi.JSONRequestDeserializer):
 
     _supported_operations = ('add', 'remove', 'replace')
 
-    def __init__(self, schema=None):
+    def __init__(self, schema=None, location_schema=None):
         super(RequestDeserializer, self).__init__()
         self.schema = schema or get_schema()
+        self.location_schema = location_schema or get_location_schema()
 
     def _get_request_body(self, request):
         output = super(RequestDeserializer, self).default(request)
@@ -1550,15 +1683,26 @@ class RequestDeserializer(wsgi.JSONRequestDeserializer):
         self._validate_import_body(body)
         return {'body': body}
 
+    def add_location(self, request):
+        body = self._get_request_body(request)
+        values = {'add_location': body}
+        try:
+            self.location_schema.validate(values)
+        except exception.InvalidObject as e:
+            raise webob.exc.HTTPBadRequest(explanation=e.msg)
+
+        return {'body': body}
+
 
 class ResponseSerializer(wsgi.JSONResponseSerializer):
     # These properties will be filtered out from the response and not
     # exposed to the client
     _hidden_properties = ['os_glance_stage_host']
 
-    def __init__(self, schema=None):
+    def __init__(self, schema=None, location_schema=None):
         super(ResponseSerializer, self).__init__()
         self.schema = schema or get_schema()
+        self.location_schema = location_schema or get_location_schema()
 
     def _get_image_href(self, image, subcollection=''):
         base_href = '/v2/images/%s' % image.image_id
@@ -1692,6 +1836,9 @@ class ResponseSerializer(wsgi.JSONResponseSerializer):
     def import_image(self, response, result):
         response.status_int = http.ACCEPTED
 
+    def add_location(self, response, result):
+        response.status_int = http.ACCEPTED
+
 
 def get_base_properties():
     return {
@@ -1885,6 +2032,53 @@ def get_base_properties():
     }
 
 
+def get_add_location_properties():
+    return {
+        'add_location': {
+            'type': 'object',
+            'description': _('Values of location url, do_secure_hash and '
+                             'validation_data for new add location API'),
+            'properties': {
+                'url': {
+                    'type': 'string',
+                    'readOnly': True,
+                    'description': _('The URL of the new location to be '
+                                     'added in the image.')
+                },
+                'validation_data': {
+                    'description': _('Values to be used to populate the '
+                                     'corresponding image properties.'
+                                     'do_secure_hash is not True then '
+                                     'image checksum and hash will not be '
+                                     'calculated so it is the responsibility'
+                                     ' of the consumer of location ADD API '
+                                     'to provide the correct values in the '
+                                     'validation_data parameter'),
+                    'type': 'object',
+                    'writeOnly': True,
+                    'additionalProperties': False,
+                    'properties': {
+                        'os_hash_algo': {
+                            'type': 'string',
+                            'maxLength': 64,
+                            'enum': ['sha1', 'sha256', 'sha512', 'md5'],
+                        },
+                        'os_hash_value': {
+                            'type': 'string',
+                            'maxLength': 128,
+                        },
+                    },
+                    'dependentRequired': {
+                        "os_hash_value": ["os_hash_algo"],
+                        "os_hash_algo": ["os_hash_value"]
+                    },
+                },
+            },
+            'required': ['url'],
+        },
+    }
+
+
 def _get_base_links():
     return [
         {'rel': 'self', 'href': '{self}'},
@@ -1896,10 +2090,7 @@ def _get_base_links():
 def get_schema(custom_properties=None):
     properties = get_base_properties()
     links = _get_base_links()
-    if CONF.allow_additional_image_properties:
-        schema = glance.schema.PermissiveSchema('image', properties, links)
-    else:
-        schema = glance.schema.Schema('image', properties)
+    schema = glance.schema.PermissiveSchema('image', properties, links)
 
     if custom_properties:
         for property_value in custom_properties.values():
@@ -1908,6 +2099,12 @@ def get_schema(custom_properties=None):
     return schema
 
 
+def get_location_schema():
+    properties = get_add_location_properties()
+    schema = glance.schema.PermissiveSchema('location', properties)
+    return schema
+
+
 def get_collection_schema(custom_properties=None):
     image_schema = get_schema(custom_properties)
     return glance.schema.CollectionSchema('images', image_schema)
@@ -1931,7 +2128,8 @@ def load_custom_properties():
 def create_resource(custom_properties=None):
     """Images resource factory method"""
     schema = get_schema(custom_properties)
-    deserializer = RequestDeserializer(schema)
-    serializer = ResponseSerializer(schema)
+    location_schema = get_location_schema()
+    deserializer = RequestDeserializer(schema, location_schema)
+    serializer = ResponseSerializer(schema, location_schema)
     controller = ImagesController()
     return wsgi.Resource(controller, deserializer, serializer)
