@@ -23,7 +23,6 @@ from oslo_config import cfg
 from oslo_log import log as logging
 from oslo_utils import encodeutils
 from oslo_utils import excutils
-import six
 from stevedore import named
 from taskflow.patterns import linear_flow as lf
 from taskflow import retry
@@ -182,6 +181,16 @@ class _ImportToFS(task.Task):
                                                'bfile': backing_file}
             raise RuntimeError(msg)
 
+        try:
+            data_file = metadata['format-specific']['data']['data-file']
+        except KeyError:
+            data_file = None
+        if data_file is not None:
+            msg = _("File %(path)s has invalid data-file "
+                    "%(dfile)s, aborting.") % {"path": path,
+                                               "dfile": data_file}
+            raise RuntimeError(msg)
+
         return path
 
     def revert(self, image_id, result, **kwargs):
@@ -289,7 +298,7 @@ class _ImportToStore(task.Task):
 
         # NOTE(flaper87): Let's dance... and fall
         #
-        # Unfortunatelly, because of the way our domain layers work and
+        # Unfortunately, because of the way our domain layers work and
         # the checks done in the FS store, we can't simply rename the file
         # and set the location. To do that, we'd have to duplicate the logic
         # of every and each of the domain factories (quota, location, etc)
@@ -388,12 +397,12 @@ class _CompleteTask(task.Task):
             # necessary
             log_msg = _LE("Task ID %(task_id)s failed. Error: %(exc_type)s: "
                           "%(e)s")
-            LOG.exception(log_msg, {'exc_type': six.text_type(type(e)),
+            LOG.exception(log_msg, {'exc_type': str(type(e)),
                                     'e': encodeutils.exception_to_unicode(e),
                                     'task_id': task.task_id})
 
             err_msg = _("Error: %(exc_type)s: %(e)s")
-            task.fail(err_msg % {'exc_type': six.text_type(type(e)),
+            task.fail(err_msg % {'exc_type': str(type(e)),
                                  'e': encodeutils.exception_to_unicode(e)})
         finally:
             self.task_repo.save(task)
