@@ -22,7 +22,6 @@ from oslo_config import cfg
 from oslo_log import log as logging
 from oslo_utils import excutils
 from oslo_utils import importutils
-import six
 
 from glance.common import exception
 from glance.common import timeutils
@@ -138,6 +137,7 @@ class Image(object):
         extra_properties = kwargs.pop('extra_properties', {})
         self.extra_properties = ExtraProperties(extra_properties)
         self.tags = kwargs.pop('tags', [])
+        self.member = kwargs.pop('member', None)
         if kwargs:
             message = _("__init__() got unexpected keyword argument '%s'")
             raise TypeError(message % list(kwargs.keys())[0])
@@ -258,7 +258,7 @@ class Image(object):
         if self.status == 'active':
             self.status = 'deactivated'
         elif self.status == 'deactivated':
-            # Noop if already deactive
+            # Noop if already deactivate
             pass
         else:
             LOG.debug("Not allowed to deactivate image in status '%s'",
@@ -351,11 +351,12 @@ class ImageMemberFactory(object):
 
 
 class Task(object):
-    _supported_task_type = ('import', 'api_image_import')
+    _supported_task_type = ('import', 'api_image_import', 'location_import')
 
     _supported_task_status = ('pending', 'processing', 'success', 'failure')
 
     def __init__(self, task_id, task_type, status, owner,
+                 image_id, user_id, request_id,
                  expires_at, created_at, updated_at,
                  task_input, result, message):
 
@@ -379,6 +380,9 @@ class Task(object):
         self.task_input = task_input
         self.result = result
         self.message = message
+        self.image_id = image_id
+        self.request_id = request_id
+        self.user_id = user_id
 
     @property
     def status(self):
@@ -391,9 +395,9 @@ class Task(object):
     @message.setter
     def message(self, message):
         if message:
-            self._message = six.text_type(message)
+            self._message = str(message)
         else:
-            self._message = six.text_type('')
+            self._message = ''
 
     def _validate_task_status_transition(self, cur_status, new_status):
         valid_transitions = {
@@ -449,7 +453,8 @@ class Task(object):
 class TaskStub(object):
 
     def __init__(self, task_id, task_type, status, owner,
-                 expires_at, created_at, updated_at):
+                 expires_at, created_at, updated_at,
+                 image_id, user_id, request_id):
         self.task_id = task_id
         self._status = status
         self.type = task_type
@@ -457,6 +462,9 @@ class TaskStub(object):
         self.expires_at = expires_at
         self.created_at = created_at
         self.updated_at = updated_at
+        self.image_id = image_id
+        self.request_id = request_id
+        self.user_id = user_id
 
     @property
     def status(self):
@@ -465,8 +473,8 @@ class TaskStub(object):
 
 class TaskFactory(object):
 
-    def new_task(self, task_type, owner,
-                 task_input=None, **kwargs):
+    def new_task(self, task_type, owner, image_id, user_id,
+                 request_id, task_input=None, **kwargs):
         task_id = str(uuid.uuid4())
         status = 'pending'
         # Note(nikhil): expires_at would be set on the task, only when it
@@ -479,12 +487,15 @@ class TaskFactory(object):
             task_type,
             status,
             owner,
+            image_id,
+            user_id,
+            request_id,
             expires_at,
             created_at,
             updated_at,
             task_input,
             kwargs.get('result'),
-            kwargs.get('message')
+            kwargs.get('message'),
         )
 
 
@@ -510,7 +521,7 @@ class TaskExecutorFactory(object):
                 if not TaskExecutorFactory.eventlet_deprecation_warned:
                     msg = _LW("The `eventlet` executor has been deprecated. "
                               "Use `taskflow` instead.")
-                    LOG.warn(msg)
+                    LOG.warning(msg)
                     TaskExecutorFactory.eventlet_deprecation_warned = True
                 task_executor = 'taskflow'
 
@@ -526,7 +537,7 @@ class TaskExecutorFactory(object):
         except ImportError:
             with excutils.save_and_reraise_exception():
                 LOG.exception(_LE("Failed to load the %s executor provided "
-                                  "in the config.") % CONF.task.task_executor)
+                                  "in the config."), CONF.task.task_executor)
 
 
 class MetadefNamespace(object):
