@@ -28,6 +28,7 @@ from oslo_utils import excutils
 
 from glance.common import exception
 from glance.common import format_inspector
+from glance.common import store_utils
 from glance.common import utils
 import glance.domain.proxy
 from glance.i18n import _, _LE, _LI, _LW
@@ -42,6 +43,7 @@ class ImageRepoProxy(glance.domain.proxy.Repo):
     def __init__(self, image_repo, context, store_api, store_utils):
         self.context = context
         self.store_api = store_api
+        self.image_repo = image_repo
         proxy_kwargs = {'context': context, 'store_api': store_api,
                         'store_utils': store_utils}
         super(ImageRepoProxy, self).__init__(image_repo,
@@ -51,7 +53,7 @@ class ImageRepoProxy(glance.domain.proxy.Repo):
         self.db_api = glance.db.get_api()
 
     def _set_acls(self, image):
-        public = image.visibility == 'public'
+        public = image.visibility in ['public', 'community']
         member_ids = []
         if image.locations and not public:
             member_repo = _get_member_repo_for_store(image,
@@ -61,8 +63,19 @@ class ImageRepoProxy(glance.domain.proxy.Repo):
             member_ids = [m.member_id for m in member_repo.list()]
         for location in image.locations:
             if CONF.enabled_backends:
+                # NOTE(whoami-rajat): Do not set_acls if store is not defined
+                # on this node. This is possible in case of edge deployment
+                # that image location is present but the actual store is
+                # not related to this node.
+                image_store = location['metadata'].get('store')
+                if image_store not in CONF.enabled_backends:
+                    msg = (_("Store %s is not available on "
+                             "this node, skipping `_set_acls` "
+                             "call.") % image_store)
+                    LOG.debug(msg)
+                    continue
                 self.store_api.set_acls_for_multi_store(
-                    location['url'], location['metadata'].get('store'),
+                    location['url'], image_store,
                     public=public, read_tenants=member_ids,
                     context=self.context
                 )
@@ -81,6 +94,22 @@ class ImageRepoProxy(glance.domain.proxy.Repo):
         self._set_acls(image)
         return result
 
+    def get(self, image_id):
+        image = super(ImageRepoProxy, self).get(image_id)
+        if CONF.enabled_backends:
+            try:
+                store_utils.update_store_in_locations(
+                    self.context, image, self.image_repo)
+            except exception.Forbidden:
+                # NOTE(danms): We may not be able to complete a store
+                # update if we do not own the image. That should not
+                # break us, so avoid raising Forbidden in that
+                # case. Note that modifications to @image here will
+                # still be returned to the user, just not saved in the
+                # DB. That is probably what we want anyway.
+                pass
+        return image
+
 
 def _get_member_repo_for_store(image, context, db_api, store_api):
     image_member_repo = glance.db.ImageMemberRepo(context, db_api, image)
@@ -566,15 +595,23 @@ class ImageProxy(glance.domain.proxy.Image):
 
         self._upload_to_store(data, verifier, backend, size)
 
-        if fmt and fmt.format_match and fmt.virtual_size:
-            self.image.virtual_size = fmt.virtual_size
-            LOG.info('Image format matched and virtual size computed: %i',
-                     self.image.virtual_size)
+        virtual_size = 0
+        if fmt and fmt.format_match:
+            try:
+                virtual_size = fmt.virtual_size
+                LOG.info('Image format matched and virtual size computed: %i',
+                         virtual_size)
+            except Exception as e:
+                LOG.error(_LE('Unable to determine virtual_size because: %s'),
+                          e)
         elif fmt:
             LOG.warning('Image format %s did not match; '
                         'unable to calculate virtual size',
                         self.image.disk_format)
 
+        if virtual_size:
+            self.image.virtual_size = fmt.virtual_size
+
         if set_active and self.image.status != 'active':
             self.image.status = 'active'
 
@@ -604,10 +641,10 @@ class ImageProxy(glance.domain.proxy.Image):
 
                 return data
             except Exception as e:
-                LOG.warn(_LW('Get image %(id)s data failed: '
-                             '%(err)s.'),
-                         {'id': self.image.image_id,
-                          'err': encodeutils.exception_to_unicode(e)})
+                LOG.warning(_LW('Get image %(id)s data failed: '
+                                '%(err)s.'),
+                            {'id': self.image.image_id,
+                            'err': encodeutils.exception_to_unicode(e)})
                 err = e
         # tried all locations
         LOG.error(_LE('Glance tried all active locations to get data for '
@@ -624,13 +661,24 @@ class ImageMemberRepoProxy(glance.domain.proxy.Repo):
         super(ImageMemberRepoProxy, self).__init__(repo)
 
     def _set_acls(self):
-        public = self.image.visibility == 'public'
+        public = self.image.visibility in ['public', 'community']
         if self.image.locations and not public:
             member_ids = [m.member_id for m in self.repo.list()]
             for location in self.image.locations:
                 if CONF.enabled_backends:
+                    # NOTE(whoami-rajat): Do not set_acls if store is not
+                    # defined on this node. This is possible in case of edge
+                    # deployment that image location is present but the actual
+                    # store is not related to this node.
+                    image_store = location['metadata'].get('store')
+                    if image_store not in CONF.enabled_backends:
+                        msg = (_("Store %s is not available on "
+                                 "this node, skipping `_set_acls` "
+                                 "call.") % image_store)
+                        LOG.debug(msg)
+                        continue
                     self.store_api.set_acls_for_multi_store(
-                        location['url'], location['metadata'].get('store'),
+                        location['url'], image_store,
                         public=public, read_tenants=member_ids,
                         context=self.context
                     )
