@@ -17,18 +17,16 @@
 
 import datetime
 import gettext
+import http.client as http
 import os
 import socket
+from unittest import mock
 
-from babel import localedata
 import eventlet.patcher
 import fixtures
-import mock
 from oslo_concurrency import processutils
 from oslo_serialization import jsonutils
 import routes
-import six
-from six.moves import http_client as http
 import webob
 
 from glance.api.v2 import router as router_v2
@@ -36,6 +34,7 @@ from glance.common import exception
 from glance.common import utils
 from glance.common import wsgi
 from glance import i18n
+from glance.image_cache import prefetcher
 from glance.tests import utils as test_utils
 
 
@@ -45,12 +44,6 @@ class RequestTest(test_utils.BaseTestCase):
         if all_locales is None:
             all_locales = []
 
-        # Override localedata.locale_identifiers to return some locales.
-        def returns_some_locales(*args, **kwargs):
-            return all_locales
-
-        self.stubs.Set(localedata, 'locale_identifiers', returns_some_locales)
-
         # Override gettext.find to return other than None for some languages.
         def fake_gettext_find(lang_id, *args, **kwargs):
             found_ret = '/glance/%s/LC_MESSAGES/glance.mo' % lang_id
@@ -62,7 +55,7 @@ class RequestTest(test_utils.BaseTestCase):
                 return found_ret
             return None
 
-        self.stubs.Set(gettext, 'find', fake_gettext_find)
+        self.mock_object(gettext, 'find', fake_gettext_find)
 
     def test_content_range(self):
         request = wsgi.Request.blank('/tests/123')
@@ -109,15 +102,9 @@ class RequestTest(test_utils.BaseTestCase):
         self.assertEqual("application/json", result)
 
     def test_params(self):
-        if six.PY2:
-            expected = webob.multidict.NestedMultiDict({
-                'limit': '20', 'name':
-                    '\xd0\x9f\xd1\x80\xd0\xb8\xd0\xb2\xd0\xb5\xd1\x82',
-                'sort_key': 'name', 'sort_dir': 'asc'})
-        else:
-            expected = webob.multidict.NestedMultiDict({
-                'limit': '20', 'name': 'Привет', 'sort_key': 'name',
-                'sort_dir': 'asc'})
+        expected = webob.multidict.NestedMultiDict({
+            'limit': '20', 'name': 'Привет', 'sort_key': 'name',
+            'sort_dir': 'asc'})
 
         request = wsgi.Request.blank("/?limit=20&name=%D0%9F%D1%80%D0%B8"
                                      "%D0%B2%D0%B5%D1%82&sort_key=name"
@@ -183,31 +170,20 @@ class RequestTest(test_utils.BaseTestCase):
         req = wsgi.Request.blank('/', headers={'Accept-Language': 'unknown'})
         self.assertIsNone(req.best_match_language())
 
-    def test_best_match_language_unknown(self):
-        # Test that we are actually invoking language negotiation by webop
+    @mock.patch.object(webob.acceptparse.AcceptLanguageValidHeader, 'lookup')
+    def test_best_match_language_unknown(self, mock_lookup):
+        # Test that we are actually invoking language negotiation by WebOb
         request = wsgi.Request.blank('/')
         accepted = 'unknown-lang'
         request.headers = {'Accept-Language': accepted}
 
-        # TODO(rosmaita): simplify when lower_constraints has webob >= 1.8.1
-        try:
-            from webob.acceptparse import AcceptLanguageValidHeader  # noqa
-            cls = webob.acceptparse.AcceptLanguageValidHeader
-            funcname = 'lookup'
-            # Bug #1765748: see comment in code in the function under test
-            # to understand why this is the correct return value for the
-            # webob 1.8.x mock
-            retval = 'fake_LANG'
-        except ImportError:
-            cls = webob.acceptparse.AcceptLanguage
-            funcname = 'best_match'
-            retval = None
+        # Bug #1765748: see comment in code in the function under test
+        # to understand why this is the correct return value for the
+        # webob 1.8.x mock
+        mock_lookup.return_value = 'fake_LANG'
 
-        with mock.patch.object(cls, funcname) as mocked_function:
-            mocked_function.return_value = retval
-
-            self.assertIsNone(request.best_match_language())
-            mocked_function.assert_called_once()
+        self.assertIsNone(request.best_match_language())
+        mock_lookup.assert_called_once()
 
         # If Accept-Language is missing or empty, match should be None
         request.headers = {'Accept-Language': ''}
@@ -325,6 +301,19 @@ class ResourceTest(test_utils.BaseTestCase):
         self.assertRaises(AttributeError, resource.dispatch, Controller(),
                           'index', 'on', pants='off')
 
+    def test_dispatch_raises_bad_request(self):
+        class FakeController(object):
+            def index(self, shirt, pants=None):
+                return (shirt, pants)
+        resource = wsgi.Resource(FakeController(), None, None)
+
+        def dispatch(self, obj, action, *args, **kwargs):
+            raise exception.InvalidPropertyProtectionConfiguration()
+        self.mock_object(wsgi.Resource, 'dispatch', dispatch)
+        request = wsgi.Request.blank('/')
+        self.assertRaises(webob.exc.HTTPBadRequest, resource.__call__,
+                          request)
+
     def test_call(self):
         class FakeController(object):
             def index(self, shirt, pants=None):
@@ -338,7 +327,7 @@ class ResourceTest(test_utils.BaseTestCase):
             if isinstance(obj, wsgi.JSONResponseSerializer):
                 raise webob.exc.HTTPForbidden()
 
-        self.stubs.Set(wsgi.Resource, 'dispatch', dispatch)
+        self.mock_object(wsgi.Resource, 'dispatch', dispatch)
 
         request = wsgi.Request.blank('/')
 
@@ -357,7 +346,7 @@ class ResourceTest(test_utils.BaseTestCase):
         def dispatch(self, obj, action, *args, **kwargs):
             raise Exception("test exception")
 
-        self.stubs.Set(wsgi.Resource, 'dispatch', dispatch)
+        self.mock_object(wsgi.Resource, 'dispatch', dispatch)
 
         request = wsgi.Request.blank('/')
 
@@ -388,33 +377,24 @@ class ResourceTest(test_utils.BaseTestCase):
                               resource, request)
         self.assertEqual(message_es, str(e))
 
+    @mock.patch.object(webob.acceptparse.AcceptLanguageValidHeader, 'lookup')
     @mock.patch.object(i18n, 'translate')
-    def test_translate_exception(self, mock_translate):
-        # TODO(rosmaita): simplify when lower_constraints has webob >= 1.8.1
-        try:
-            from webob.acceptparse import AcceptLanguageValidHeader  # noqa
-            cls = webob.acceptparse.AcceptLanguageValidHeader
-            funcname = 'lookup'
-        except ImportError:
-            cls = webob.acceptparse.AcceptLanguage
-            funcname = 'best_match'
+    def test_translate_exception(self, mock_translate, mock_lookup):
+        mock_translate.return_value = 'No Encontrado'
+        mock_lookup.return_value = 'de'
 
-        with mock.patch.object(cls, funcname) as mocked_function:
-            mock_translate.return_value = 'No Encontrado'
-            mocked_function.return_value = 'de'
+        req = wsgi.Request.blank('/tests/123')
+        req.headers["Accept-Language"] = "de"
 
-            req = wsgi.Request.blank('/tests/123')
-            req.headers["Accept-Language"] = "de"
-
-            e = webob.exc.HTTPNotFound(explanation='Not Found')
-            e = wsgi.translate_exception(req, e)
-            self.assertEqual('No Encontrado', e.explanation)
+        e = webob.exc.HTTPNotFound(explanation='Not Found')
+        e = wsgi.translate_exception(req, e)
+        self.assertEqual('No Encontrado', e.explanation)
 
     def test_response_headers_encoded(self):
         # prepare environment
-        for_openstack_comrades =  \
-            u'\u0417\u0430 \u043e\u043f\u0435\u043d\u0441\u0442\u0435\u043a, ' \
-            u'\u0442\u043e\u0432\u0430\u0440\u0438\u0449\u0438'
+        for_openstack_comrades = (
+            '\u0417\u0430 \u043e\u043f\u0435\u043d\u0441\u0442\u0435\u043a, '
+            '\u0442\u043e\u0432\u0430\u0440\u0438\u0449\u0438')
 
         class FakeController(object):
             def index(self, shirt, pants=None):
@@ -432,8 +412,7 @@ class ResourceTest(test_utils.BaseTestCase):
         response = resource.__call__(request)
 
         # ensure it has been encoded correctly
-        value = (response.headers['unicode_test'].decode('utf-8')
-                 if six.PY2 else response.headers['unicode_test'])
+        value = response.headers['unicode_test']
         self.assertEqual(for_openstack_comrades, value)
 
 
@@ -580,13 +559,28 @@ class JSONRequestDeserializerTest(test_utils.BaseTestCase):
 
 
 class ServerTest(test_utils.BaseTestCase):
-    def test_create_pool(self):
+    @mock.patch.object(prefetcher, 'Prefetcher')
+    def test_create_pool(self, mock_prefetcher):
         """Ensure the wsgi thread pool is an eventlet.greenpool.GreenPool."""
         actual = wsgi.Server(threads=1).create_pool()
         self.assertIsInstance(actual, eventlet.greenpool.GreenPool)
 
+    @mock.patch.object(prefetcher, 'Prefetcher')
     @mock.patch.object(wsgi.Server, 'configure_socket')
-    def test_http_keepalive(self, mock_configure_socket):
+    def test_reserved_stores_not_allowed(self, mock_configure_socket,
+                                         mock_prefetcher):
+        """Ensure the reserved stores are not allowed"""
+        enabled_backends = {'os_glance_file_store': 'file'}
+        self.config(enabled_backends=enabled_backends)
+        server = wsgi.Server(threads=1, initialize_glance_store=True)
+        self.assertRaises(RuntimeError, server.configure)
+
+    @mock.patch.object(prefetcher, 'Prefetcher')
+    @mock.patch.object(wsgi.Server, 'configure_socket')
+    @mock.patch('glance.sqlite_migration.can_migrate_to_central_db')
+    def test_http_keepalive(self, mock_migrate_db, mock_configure_socket,
+                            mock_prefetcher):
+        mock_migrate_db.return_value = False
         self.config(http_keepalive=False)
         self.config(workers=0)
 
@@ -607,8 +601,14 @@ class ServerTest(test_utils.BaseTestCase):
                                                 keepalive=False,
                                                 socket_timeout=900)
 
-    def test_number_of_workers(self):
+    @mock.patch.object(prefetcher, 'Prefetcher')
+    @mock.patch('glance.sqlite_migration.can_migrate_to_central_db')
+    def test_number_of_workers_posix(self, mock_migrate_db, mock_prefetcher):
         """Ensure the number of workers matches num cpus limited to 8."""
+        mock_migrate_db.return_value = False
+        if os.name == 'nt':
+            raise self.skipException("Unsupported platform.")
+
         def pid():
             i = 1
             while True:
@@ -642,6 +642,34 @@ class ServerTest(test_utils.BaseTestCase):
             self.assertEqual(expected_workers,
                              len(server.children))
 
+    @mock.patch('glance.sqlite_migration.can_migrate_to_central_db')
+    def test_invalid_staging_uri(self, mock_migrate_db):
+        mock_migrate_db.return_value = False
+        self.config(node_staging_uri='http://good.luck')
+        server = wsgi.Server()
+        with mock.patch.object(server, 'start_wsgi'):
+            # Make sure a stating URI with an bad scheme will abort startup
+            self.assertRaises(exception.GlanceException,
+                              server.start, 'fake-application', 34567)
+
+    @mock.patch('os.path.exists')
+    @mock.patch('glance.sqlite_migration.can_migrate_to_central_db')
+    def test_missing_staging_dir(self, mock_migrate_db, mock_exists):
+        mock_migrate_db.return_value = False
+        mock_exists.return_value = False
+        server = wsgi.Server()
+        with mock.patch.object(server, 'start_wsgi'):
+            # Since we are mocking out start_wsgi, create a fake pool ourselves
+            server.pool = mock.MagicMock()
+            with mock.patch.object(wsgi, 'LOG') as mock_log:
+                server.start('fake-application', 34567)
+                mock_exists.assert_called_once_with('/tmp/staging/')
+                # Make sure a missing staging directory will log a warning.
+                mock_log.warning.assert_called_once_with(
+                    'Import methods are enabled but staging directory '
+                    '%(path)s does not exist; Imports will fail!',
+                    {'path': '/tmp/staging/'})
+
 
 class TestHelpers(test_utils.BaseTestCase):
 
@@ -660,8 +688,8 @@ class TestHelpers(test_utils.BaseTestCase):
                    'location': "file:///tmp/glance-tests/2",
                    'properties': {'distro': 'Ubuntu 10.04 LTS'}}
         headers = utils.image_meta_to_http_headers(fixture)
-        for k, v in six.iteritems(headers):
-            self.assertIsInstance(v, six.text_type)
+        for k, v in headers.items():
+            self.assertIsInstance(v, str)
 
     def test_data_passed_properly_through_headers(self):
         """
@@ -681,7 +709,7 @@ class TestHelpers(test_utils.BaseTestCase):
         response = FakeResponse()
         response.headers = headers
         result = utils.get_image_meta_from_headers(response)
-        for k, v in six.iteritems(fixture):
+        for k, v in fixture.items():
             if v is not None:
                 self.assertEqual(v, result[k])
             else:
@@ -704,19 +732,11 @@ class GetSocketTestCase(test_utils.BaseTestCase):
         self.useFixture(fixtures.MonkeyPatch(
             "glance.common.wsgi.time.time",
             mock.Mock(side_effect=[0, 1, 5, 10, 20, 35])))
-        self.useFixture(fixtures.MonkeyPatch(
-            "glance.common.wsgi.utils.validate_key_cert",
-            lambda *x: None))
-        wsgi.CONF.cert_file = '/etc/ssl/cert'
-        wsgi.CONF.key_file = '/etc/ssl/key'
-        wsgi.CONF.ca_file = '/etc/ssl/ca_cert'
         wsgi.CONF.tcp_keepidle = 600
 
-    def test_correct_configure_socket(self):
+    @mock.patch.object(prefetcher, 'Prefetcher')
+    def test_correct_configure_socket(self, mock_prefetcher):
         mock_socket = mock.Mock()
-        self.useFixture(fixtures.MonkeyPatch(
-            'glance.common.wsgi.ssl.wrap_socket',
-            mock_socket))
         self.useFixture(fixtures.MonkeyPatch(
             'glance.common.wsgi.eventlet.listen',
             lambda *x, **y: mock_socket))
@@ -732,23 +752,16 @@ class GetSocketTestCase(test_utils.BaseTestCase):
             socket.SO_KEEPALIVE,
             1), mock_socket.mock_calls)
         if hasattr(socket, 'TCP_KEEPIDLE'):
-            self.assertIn(mock.call().setsockopt(
+            self.assertIn(mock.call.setsockopt(
                 socket.IPPROTO_TCP,
                 socket.TCP_KEEPIDLE,
                 wsgi.CONF.tcp_keepidle), mock_socket.mock_calls)
 
-    def test_get_socket_without_all_ssl_reqs(self):
-        wsgi.CONF.key_file = None
-        self.assertRaises(RuntimeError, wsgi.get_socket, 1234)
-
     def test_get_socket_with_bind_problems(self):
         self.useFixture(fixtures.MonkeyPatch(
             'glance.common.wsgi.eventlet.listen',
             mock.Mock(side_effect=(
                 [wsgi.socket.error(socket.errno.EADDRINUSE)] * 3 + [None]))))
-        self.useFixture(fixtures.MonkeyPatch(
-            'glance.common.wsgi.ssl.wrap_socket',
-            lambda *x, **y: None))
 
         self.assertRaises(RuntimeError, wsgi.get_socket, 1234)
 
@@ -756,9 +769,6 @@ class GetSocketTestCase(test_utils.BaseTestCase):
         self.useFixture(fixtures.MonkeyPatch(
             'glance.common.wsgi.eventlet.listen',
             mock.Mock(side_effect=wsgi.socket.error(socket.errno.ENOMEM))))
-        self.useFixture(fixtures.MonkeyPatch(
-            'glance.common.wsgi.ssl.wrap_socket',
-            lambda *x, **y: None))
         self.assertRaises(wsgi.socket.error, wsgi.get_socket, 1234)
 
 
