@@ -44,7 +44,7 @@ class ImageRepoStub(object):
 
 class ImageStub(object):
     def __init__(self, image_id, status=None, locations=None,
-                 visibility=None, extra_properties=None):
+                 visibility=None, extra_properties=None, virtual_size=0):
         self.image_id = image_id
         self.status = status
         self.locations = locations or []
@@ -56,7 +56,7 @@ class ImageStub(object):
         self.checksum = None
         self.disk_format = 'raw'
         self.container_format = 'bare'
-        self.virtual_size = 0
+        self.virtual_size = virtual_size
 
     def delete(self):
         self.status = 'deleted'
@@ -112,7 +112,8 @@ class TestStoreMultiBackends(utils.BaseTestCase):
             'img_signature': 'VALID'
         }
         image_stub = ImageStub(UUID2, status='queued', locations=[],
-                               extra_properties=extra_properties)
+                               extra_properties=extra_properties,
+                               virtual_size=4)
         image_stub.disk_format = 'iso'
         image = glance.location.ImageProxy(image_stub, context,
                                            self.store_api, self.store_utils)
@@ -283,6 +284,37 @@ class TestStoreImage(utils.BaseTestCase):
         self.assertEqual('active', image.status)
         self.assertEqual(0, image.virtual_size)
 
+    @mock.patch('glance.common.format_inspector.QcowInspector.virtual_size',
+                new_callable=mock.PropertyMock)
+    @mock.patch('glance.common.format_inspector.QcowInspector.format_match',
+                new_callable=mock.PropertyMock)
+    def test_image_set_data_inspector_virtual_size_failure(self, mock_fm,
+                                                           mock_vs):
+        # Force our format to match
+        mock_fm.return_value = True
+
+        # Make virtual_size fail in some unexpected way
+        mock_vs.side_effect = ValueError('some error')
+
+        context = glance.context.RequestContext(user=USER1)
+        image_stub = ImageStub(UUID2, status='queued', locations=[])
+        image_stub.disk_format = 'qcow2'
+        # We are going to pass an iterable data source, so use the
+        # FakeStoreAPIReader that actually reads from that data
+        store_api = unit_test_utils.FakeStoreAPIReader()
+        image = glance.location.ImageProxy(image_stub, context,
+                                           store_api, self.store_utils)
+
+        # Make sure set_data proceeds even though the format clearly
+        # does not match
+        image.set_data(iter(['YYYY']), 4)
+        self.assertEqual(4, image.size)
+        # NOTE(markwash): FakeStore returns image_id for location
+        self.assertEqual(UUID2, image.locations[0]['url'])
+        self.assertEqual('Z', image.checksum)
+        self.assertEqual('active', image.status)
+        self.assertEqual(0, image.virtual_size)
+
     @mock.patch('glance.common.format_inspector.get_inspector')
     def test_image_set_data_inspector_not_needed(self, mock_gi):
         context = glance.context.RequestContext(user=USER1)
